@@ -24,20 +24,37 @@ rmulti <- function(m,s,kappa) {
 }
 
 set.seed(1234)
+lam <- 1000
 sigma <- matrix(c(0.1,0.1,0.1,0.25), ncol=2)
-theta <- list("lam"=500,"rmulti"=list("m"=c(-3.0,-2.0),"s"=sigma,"kappa"=0.5))
-S <- simSpheroidSystem(theta,rjoint="rmulti",box=list(c(0,5)),pl=101)
+theta <- list("m"=c(-3.0,-2.0),"s"=sigma,"kappa"=0.5)
+S <- simSpheroidSystem(theta,lam,rjoint="rmulti",box=list(c(0,5)),pl=101)
 
-# Spheroids with lognormal distributed length of major axis
-theta <- list("lam"=1000,"size"=list("meanlog"=-2.5,"sdlog"=0.5),
+# Spheroids with log normal distributed semi-major axis length
+# distribution and independent orientation distribution
+theta <- list("size"=list("meanlog"=-2.5,"sdlog"=0.5),
               "shape"=list("s"=0.5),
               "orientation"=list("kappa"=1.5))
-S <- simSpheroidSystem(theta,size="rlnorm",orientation="rbetaiso",
+S <- simSpheroidSystem(theta,lam,size="rlnorm",orientation="rbetaiso",
                        box=list(c(0,5)),pl=101)
 
 # Spheroids of constant sizes
-theta <- list("lam"=1000,"size"=list(0.25),"shape"=list("s"=0.5),
+theta <- list("size"=list(0.25),
+              "shape"=list("s"=0.5),
               "orientation"=list("kappa"=1))
-S <- simSpheroidSystem(theta,size="const",orientation="rbetaiso",
+S <- simSpheroidSystem(theta,lam,size="const",orientation="rbetaiso",
                        box=list(c(0,5)),pl=101)
+			   
+			   
+# Spheroids exact simulation
+param <- list("mx"=-1.0,"my"=-2.5, "sdx"=0.15,"sdy"=0.2,"rho"=0.0,"kappa"=1.0)
+theta <- list("size"=list("mx"=param$mx,
+					      "sdx"=param$sdx,
+						  "my"=param$my,
+						  "sdy"=param$sdy,
+						  "rho"=param$rho),
+			  "orientation"=list("kappa"=param$kappa),
+			  "shape"=list())
+
+S <- simSpheroidSystem(theta,lam,size="rbinorm",orientation="rbetaiso",box=list(c(0,5)),pl=101)	
+
 }
