@@ -1,6 +1,15 @@
+/*
+ * simSphere.h
+ *
+ *  Created on: 07.05.2015
+ *      Author: franke
+ */
+
 #ifndef SRC_SIMSPHERE_H_
 #define SRC_SIMSPHERE_H_
 
+#include "Utils.h"
+#include "Rheaders.h"
 #include "Intersector.h"
 
 #ifdef __cplusplus
@@ -8,6 +17,7 @@ extern "C" {
 #endif
 
   SEXP SphereSystem(SEXP R_param, SEXP R_cond);
+  SEXP SimulateSpheresAndIntersect(SEXP R_param, SEXP R_cond);
   SEXP GetSphereSystem(SEXP ext);
   SEXP SetupSphereSystem(SEXP R_vname, SEXP R_env, SEXP R_param, SEXP R_cond);
   SEXP IntersectSphereSystem(SEXP ext, SEXP R_n, SEXP R_z);
@@ -27,14 +37,17 @@ public:
     m_box(box),m_lam(lam), num(0)
   {}
 
-    void simSphereSys(R_Calldata d);
-    size_t getNumSpheres() const { return num; }
-    STGM::Spheres &refObjects()  { return m_spheres; }
-    const STGM::Spheres &refObjects() const { return m_spheres; }
+  ~CBoolSphereSystem() {};
 
-    void IntersectWithPlane(IntersectorSpherePlaneVec &objects, STGM::CPlane &plane);
+  void simSphereSys(R_Calldata d);
+  size_t getNumSpheres() const { return num; }
+  STGM::Spheres &refObjects()  { return m_spheres; }
+  const STGM::Spheres &refObjects() const { return m_spheres; }
 
-    template<typename F> void simSpheres(F f);
+  void IntersectWithPlane(IntersectorSpherePlaneVec &objects, STGM::CPlane &plane);
+
+  template<typename F> void simSpheres(F f);
+  void simSpheresPerfect(double mx, double sdx);
 
 
 
