@@ -46,6 +46,18 @@ SEXP showArgs(SEXP args) {
 }
 
 
+
+SEXP getSingleCall(SEXP R_fname, SEXP R_arg, SEXP R_rho) {
+    SEXP RCallBack = R_NilValue;
+    PROTECT(RCallBack = allocVector(LANGSXP,2));
+    SETCAR( RCallBack, findFun(install(CHAR(STRING_ELT(R_fname, 0))),R_rho ));
+    SETCAR(CDR(RCallBack),R_arg);
+
+    UNPROTECT(1);
+    return RCallBack;
+}
+
+
 /* get a single call to an R function */
 SEXP getCall(SEXP R_fname, SEXP R_args, SEXP R_rho) {
   SEXP RCallBack = R_NilValue;
@@ -98,23 +110,6 @@ SEXP getVar(SEXP name, SEXP rho)
     return ans;
 }
 
-////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-
-extern "C" {
-
-    void real_eval(double *a, int *n, double *evalf, int *err) {
-            // size(evalf): n
-            // result: evec = a
-
-            int lda = *n,  lwork = 3*lda-1;
-            double *work = Calloc(lwork, double);
-            F77_NAME(dsyev)("V","U", &lda, a, &lda, evalf, work, &lwork, err);
-            Free(work);
-
-    }
-
-}
-
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -143,3 +138,6 @@ void * getExternalPtr(SEXP ext)
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+
+
